"""
This is a setup.py script generated by py2applet

Usage:
    python setup.py py2app
"""

from setuptools import setup, find_packages
import py2app

APP_NAME = "pomsets"
BUNDLE_NAME = APP_NAME

# NOTE:
# remember to also update the Application package
VERSION="1.0.7"

APP = ['src/pomsets_app/pomsets_qt.py']
DATA_FILES = ['LICENSE']
RESOURCES = [
    'resources/config',
    'resources/fonts',
    'resources/images',
    'resources/licenses',
    'resources/library']
OPTIONS = {
    # specify icon file
    # iconfile='resources/myapp-icon.icns',
    # specify license
    # resources=['resources/License.txt'],
    'site_packages':True,
    'argv_emulation': False, 
    'includes': ['sip'],
    'resources':RESOURCES,
    'iconfile':'resources/images/logo.icns',
    'packages':[],
    'plist':dict(
        CFBundleName=BUNDLE_NAME,
        CFBundleShortVersionString=VERSION,
        CFBundleGetInfoString="%s %s" % (APP_NAME, VERSION),
        CFBundleExecutable=APP_NAME,
        CFBundleIdentifier="org.pomsets.app"
        )
    }

"""
    import py2app

    # Build the .app file
    setup(
        options=dict(
            py2app=dict(
                iconfile='resources/myapp-icon.icns',
                packages='wx',
                site_packages=True,
                resources=['resources/License.txt'],
                plist=dict(
                    CFBundleName               = "MyAppName",
                    CFBundleShortVersionString = "0.2.5",     # must be in X.X.X format
                    CFBundleGetInfoString      = "MyAppName 0.2.5",
                    CFBundleExecutable         = "MyAppName",
                    CFBundleIdentifier         = "com.example.myappname",
                ),
            ),
        ),
        app=[ 'myapp.py' ]
    )

"""



setup(
    name='pomsets-gui',
    version=VERSION,
    packages=find_packages(
        'src',exclude=["*.test", "*.test.*", "test.*", "test",
                       "*.utils", "*.utils.*", "utils.*", "utils",
                       "*.wx", "*.wx.*", "wx.*", "wx",
                       ]),
    package_dir={'':'src'},
    test_suite="test",
    tests_require=[
        "euca2ools>=1.1"
        ],
    install_requires = [
        'pypatterns>=0.1.1',
        'pomsets-core>=1.0.6',
        'PyOpenGL',
        ],
    app=APP,
    data_files=DATA_FILES,
    options={'py2app': OPTIONS},
    setup_requires=['py2app'],


    # metadata for upload to PyPI
    author = "michael j pan",
    author_email = "mikepan@gmail.com",
    description = "GUI for workflow management for the cloud",
    license = "GPL (personal use) or commercial",
    keywords = "workflow cloud hadoop parameter sweep",
    url = "http://pomsets.org",
    long_description = "pomsets-gui implements the GUI for workflow management in the cloud.",
    platforms = ["All"]

)
