# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/edit definition dialog.ui'
#
# Created: Thu Mar  4 16:11:06 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(409, 683)
        Dialog.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(90, 640, 291, 32))
        self.buttonBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Apply|QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.checkBoxStageable = QtGui.QCheckBox(Dialog)
        self.checkBoxStageable.setGeometry(QtCore.QRect(130, 180, 121, 20))
        self.checkBoxStageable.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkBoxStageable.setObjectName("checkBoxStageable")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(21, 30, 71, 16))
        self.label.setObjectName("label")
        self.label_2 = QtGui.QLabel(Dialog)
        self.label_2.setGeometry(QtCore.QRect(21, 70, 71, 16))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtGui.QLabel(Dialog)
        self.label_3.setGeometry(QtCore.QRect(20, 110, 101, 16))
        self.label_3.setObjectName("label_3")
        self.labelExecutablePath = QtGui.QLabel(Dialog)
        self.labelExecutablePath.setGeometry(QtCore.QRect(20, 150, 101, 16))
        self.labelExecutablePath.setObjectName("labelExecutablePath")
        self.labelJarFile = QtGui.QLabel(Dialog)
        self.labelJarFile.setGeometry(QtCore.QRect(20, 220, 62, 16))
        self.labelJarFile.setObjectName("labelJarFile")
        self.labelJarClass = QtGui.QLabel(Dialog)
        self.labelJarClass.setGeometry(QtCore.QRect(20, 260, 62, 16))
        self.labelJarClass.setObjectName("labelJarClass")
        self.labelStaticArgs = QtGui.QLabel(Dialog)
        self.labelStaticArgs.setGeometry(QtCore.QRect(20, 300, 71, 16))
        self.labelStaticArgs.setObjectName("labelStaticArgs")
        self.label_8 = QtGui.QLabel(Dialog)
        self.label_8.setGeometry(QtCore.QRect(20, 350, 81, 16))
        self.label_8.setObjectName("label_8")
        self.tableParameters = QtGui.QTableWidget(Dialog)
        self.tableParameters.setGeometry(QtCore.QRect(20, 380, 361, 191))
        self.tableParameters.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tableParameters.setObjectName("tableParameters")
        self.tableParameters.setColumnCount(2)
        self.tableParameters.setRowCount(0)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(1, item)
        self.buttonRemove = QtGui.QPushButton(Dialog)
        self.buttonRemove.setGeometry(QtCore.QRect(290, 580, 91, 32))
        self.buttonRemove.setObjectName("buttonRemove")
        self.buttonEdit = QtGui.QPushButton(Dialog)
        self.buttonEdit.setGeometry(QtCore.QRect(200, 580, 91, 32))
        self.buttonEdit.setObjectName("buttonEdit")
        self.buttonAdd = QtGui.QPushButton(Dialog)
        self.buttonAdd.setGeometry(QtCore.QRect(110, 580, 91, 32))
        self.buttonAdd.setObjectName("buttonAdd")
        self.line = QtGui.QFrame(Dialog)
        self.line.setGeometry(QtCore.QRect(20, 330, 361, 16))
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.line_2 = QtGui.QFrame(Dialog)
        self.line_2.setGeometry(QtCore.QRect(20, 620, 361, 16))
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.comboBoxExecutableType = QtGui.QComboBox(Dialog)
        self.comboBoxExecutableType.setGeometry(QtCore.QRect(130, 100, 251, 31))
        self.comboBoxExecutableType.setObjectName("comboBoxExecutableType")
        self.textEditName = QtGui.QLineEdit(Dialog)
        self.textEditName.setGeometry(QtCore.QRect(130, 21, 251, 31))
        self.textEditName.setObjectName("textEditName")
        self.textEditDescription = QtGui.QLineEdit(Dialog)
        self.textEditDescription.setGeometry(QtCore.QRect(130, 60, 251, 31))
        self.textEditDescription.setObjectName("textEditDescription")
        self.textEditExecutablePath = QtGui.QLineEdit(Dialog)
        self.textEditExecutablePath.setGeometry(QtCore.QRect(130, 140, 251, 31))
        self.textEditExecutablePath.setObjectName("textEditExecutablePath")
        self.textEditJarFile = QtGui.QLineEdit(Dialog)
        self.textEditJarFile.setGeometry(QtCore.QRect(130, 210, 251, 31))
        self.textEditJarFile.setObjectName("textEditJarFile")
        self.textEditJarClass = QtGui.QLineEdit(Dialog)
        self.textEditJarClass.setGeometry(QtCore.QRect(130, 250, 251, 31))
        self.textEditJarClass.setObjectName("textEditJarClass")
        self.textEditStaticArgs = QtGui.QLineEdit(Dialog)
        self.textEditStaticArgs.setGeometry(QtCore.QRect(130, 290, 251, 31))
        self.textEditStaticArgs.setObjectName("textEditStaticArgs")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Edit definition", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBoxStageable.setText(QtGui.QApplication.translate("Dialog", "Stageable", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "Description", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Dialog", "Executable type", None, QtGui.QApplication.UnicodeUTF8))
        self.labelExecutablePath.setText(QtGui.QApplication.translate("Dialog", "Executable path", None, QtGui.QApplication.UnicodeUTF8))
        self.labelJarFile.setText(QtGui.QApplication.translate("Dialog", "Jar file", None, QtGui.QApplication.UnicodeUTF8))
        self.labelJarClass.setText(QtGui.QApplication.translate("Dialog", "Jar class", None, QtGui.QApplication.UnicodeUTF8))
        self.labelStaticArgs.setText(QtGui.QApplication.translate("Dialog", "Static args", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("Dialog", "Parameters", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(0).setText(QtGui.QApplication.translate("Dialog", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(1).setText(QtGui.QApplication.translate("Dialog", "Direction", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonRemove.setText(QtGui.QApplication.translate("Dialog", "Remove", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonEdit.setText(QtGui.QApplication.translate("Dialog", "Edit", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonAdd.setText(QtGui.QApplication.translate("Dialog", "Add", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())

